package antlr.util.lout;

import antlr.*;
import antlr.collections.*;
import antlr.util.*;
import org.antlr.stringtemplate.*;

import java.io.StringReader;

/** Class grouping Lout generation utilities */
public class LoutGenerator {

	/** Allow this class to be used as a command line tool for testing */
	public static void main(String[] args) {
		if ( args.length!=1 ) {
			System.out.println("LoutGenerator tree-spec-string");
			return;
		}
		//System.out.println(treeToLout(args[0]));
		System.out.println(grammarToLout(args[0]));
	}

	/** Convert grammar to Lout

 stat : IF LPAREN expr RPAREN stat (ELSE stat)? "|" RETURN expr ;

 to

	 @SyntaxDiag
	 title { ifstat }
	 {
	 @StartRight
	 @Select
	   A { @Sequence
		 A { @BCell IF } B { @CCell ( } C { @ACell expr} D { @CCell ) }
		 E { @ACell stat }
		 F { @Optional
		   @Sequence
			 A { @BCell ELSE } B { @ACell stat }
		 }
	   }
	   B { @Sequence
		 A { @BCell RETURN } B { @ACell expr }
	   }
	 }
	 */
	public static String grammarToLout(String grammar) {
		StringTemplateGroup group = new StringTemplateGroup("loutSyndiags");
		group.setErrorListener(
				new StringTemplateErrorListener() {
					public void warning(String msg)
					{
						System.err.println("StringTemplate error: "+msg);
					}
					public void debug(String msg)
					{
						System.err.println("StringTemplate error: "+msg);
					}

					public void error(String msg, Exception e)
					{
						System.err.println("StringTemplate error: "+msg);
					}
				}
		);
		return grammarToLout(grammar,group);
	}

	public static String grammarToLout(String grammar,
									   StringTemplateGroup group)
	{
		if ( grammar==null ) {
			return null;
		}
		AST t = GrammarParser.toAST(grammar);
		if (t == null) {
            return null;
        }
		GrammarTreeToLout tp = new GrammarTreeToLout(group);
		// System.out.println("Tree is "+t.toStringList());
		try {
			String r = tp.rules(t); // translate
			return r;
		}
		catch (RecognitionException re) {
			System.err.println("problems parsing grammar: "+re.getMessage());
		}
		return null;
	}

	/** Convert a tree in lisp notation to Lout.
	 * For example, convert the tree
	 *
	 *   (IF expr stat stat)
	 *
	 * to
	 *
	 	@Diag
		  vstrut {yes}
		  treevsep {1f}
		  vmargin {.1f}
		  outlinestyle {noline}
		{
		@Tree {
		  @Box IF
		@FirstSub arrow {yes} {
		  @Box expr
		}
		@NextSub arrow {yes} {
		 @Box stat
		}
		@NextSub arrow {yes} {
		 @Box stat
		}

		}
		}
	 *
	 * This routine uses the following string templates prefixed
	 * with antlr/util/lout/...
	 *
	 * firstChild.st       rootNode.st         treeStructure.st
	 * nextChild.st        tree.st
	 */
	public static String treeToLout(String tree) {
		StringTemplateGroup group = new StringTemplateGroup("loutTrees");
		group.setErrorListener(
				new StringTemplateErrorListener() {
					public void warning(String msg)
					{
						System.err.println("StringTemplate error: "+msg);
					}
					public void debug(String msg)
					{
						System.err.println("StringTemplate error: "+msg);
					}

					public void error(String msg, Exception e) {
						System.err.println("StringTemplate error: "+msg);
					}
				}
		);
		return treeToLout(tree,
						  group.getTemplate("antlr/util/lout/tree"),
						  group.getTemplate("antlr/util/lout/treeStructure"),
						  group.getTemplate("antlr/util/lout/rootNode"),
						  group.getTemplate("antlr/util/lout/firstChild"),
						  group.getTemplate("antlr/util/lout/nextChild"));
	}

	public static String treeToLout(String tree,
									StringTemplate overallTemplate,
									StringTemplate treeStructureTemplate,
									StringTemplate rootTemplate,
									StringTemplate firstChildTemplate,
									StringTemplate nextChildTemplate)
	{
		if ( tree==null ) {
			return null;
		}
		AST t = LispTreeParser.toAST(tree);
		if (t == null) {
            return null;
        }
		String tStr = treeToLout(t,
								 1, // first child
								 treeStructureTemplate,
								 rootTemplate,
								 firstChildTemplate,
								 nextChildTemplate);
		overallTemplate.setAttribute("t", tStr);
		return overallTemplate.toString();
	}

	private static String treeToLout(AST t,
									 int n,
									 StringTemplate treeStructureTemplate,
									 StringTemplate rootTemplate,
									 StringTemplate firstChildTemplate,
									 StringTemplate nextChildTemplate)
	{
		// System.out.println("tree in AST form: "+t.toString());
		if ( t.getFirstChild()!=null ) {
			StringTemplate structST = treeStructureTemplate.getInstanceOf();

			// SET ROOT NODE
			StringTemplate rST = rootTemplate.getInstanceOf();
			rST.setAttribute("label", nodeTextToLoutLabel(t.getText()));
			structST.setAttribute("root", rST.toString());
			// System.out.println("root st is "+rST.toString());

			// SET FIRST CHILD
			AST first = t.getFirstChild();
			n=1;
			String fTree = treeToLout(first,
									  n,
									  treeStructureTemplate,
									  rootTemplate,
									  firstChildTemplate,
									  nextChildTemplate);
			structST.setAttribute("first", fTree);

			// SET NEXT CHILDREN
			AST next = first.getNextSibling();
			while ( next!=null ) {
				n++;
				String nTree = treeToLout(next,
										  n,
										  treeStructureTemplate,
										  rootTemplate,
										  firstChildTemplate,
										  nextChildTemplate);
				structST.setAttribute("otherChildren", nTree);
				next = next.getNextSibling();
			}
			return structST.toString();
		}
		else {
			if ( n==1 ) {
				StringTemplate fST = firstChildTemplate.getInstanceOf();
				fST.setAttribute("label", nodeTextToLoutLabel(t.getText()));
				// System.out.println("first leaf st is "+fST.toString());
				return fST.toString();
			}
			else {
				StringTemplate nST = nextChildTemplate.getInstanceOf();
				nST.setAttribute("label", nodeTextToLoutLabel(t.getText()));
				// System.out.println("next leaf st is "+nST.toString());
				return nST.toString();
			}
		}
	}

	private static String nodeTextToLoutLabel(String text) {
		return text;
	}
}
